@@ -7,14 +7,9 @@ use Carp 'croak', 'carp';
 require Rstats;
 use Rstats::Elements;
 use Scalar::Util ();
-use Math::Complex ();
 use Math::Trig ();
 use POSIX ();
 
-# Perl infinite values(this is value is only valid as return value)
-my $perl_inf_result = 9 ** 9 ** 9;
-my $perl_negative_inf_result = -9 ** 9 ** 9;
-
 sub TRUE () { new_true() }
 sub FALSE () { new_false() }
 sub NA () { new_na() }
@@ -1194,603 +1189,6 @@ sub or {
   }
 }
 
-sub add {
-  my ($e1, $e2) = @_;
-  
-  return NA if $e1->is_na || $e2->is_na;
-  
-  if ($e1->is_character) {
-    croak "Error in a + b : non-numeric argument to binary operator";
-  }
-  elsif ($e1->is_complex) {
-    my $re = add($e1->re, $e2->re);
-    my $im = add($e1->im, $e2->im);
-    
-    return complex_double($re, $im);
-  }
-  elsif ($e1->is_double) {
-    return NaN if $e1->is_nan || $e2->is_nan;
-    if (defined $e1->dv) {
-      if (defined $e2->dv) {
-        my $value = $e1->dv + $e2->dv;
-        if ($value == $perl_inf_result) {
-          return Inf;
-        }
-        elsif ($value == $perl_negative_inf_result) {
-          return negativeInf;
-        }
-        else {
-          return double($value)
-        }
-      }
-      elsif ($e2->is_positive_infinite) {
-        return Inf;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return negativeInf;
-      }
-    }
-    elsif ($e1->is_positive_infinite) {
-      if (defined $e2->dv) {
-        return Inf;
-      }
-      elsif ($e2->is_positive_infinite) {
-        return Inf;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return NaN;
-      }
-    }
-    elsif ($e1->is_negative_infinite) {
-      if (defined $e2->dv) {
-        return negativeInf;
-      }
-      elsif ($e2->is_positive_infinite) {
-        return NaN;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return negativeInf;
-      }
-    }
-  }
-  elsif ($e1->is_integer) {
-    return integer($e1->iv + $e2->iv);
-  }
-  elsif ($e1->is_logical) {
-    return integer($e1->iv + $e2->iv);
-  }
-  else {
-    croak "Invalid type";
-  }
-}
-
-sub subtract {
-  my ($e1, $e2) = @_;
-  
-  return NA if $e1->is_na || $e2->is_na;
-  
-  if ($e1->is_character) {
-    croak "Error in a + b : non-numeric argument to binary operator";
-  }
-  elsif ($e1->is_complex) {
-    my $re = subtract($e1->re, $e2->re);
-    my $im = subtract($e1->im, $e2->im);
-    
-    return complex_double($re, $im);
-  }
-  elsif ($e1->is_double) {
-    return NaN if $e1->is_nan || $e2->is_nan;
-    if (defined $e1->dv) {
-      if (defined $e2->dv) {
-        my $value = $e1->dv - $e2->dv;
-        if ($value == $perl_inf_result) {
-          return Inf;
-        }
-        elsif ($value == $perl_negative_inf_result) {
-          return negativeInf;
-        }
-        else {
-          return double($value)
-        }
-      }
-      elsif ($e2->is_positive_infinite) {
-        return negativeInf;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return Inf;
-      }
-    }
-    elsif ($e1->is_positive_infinite) {
-      if (defined $e2->dv) {
-        return Inf;
-      }
-      elsif ($e2->is_positive_infinite) {
-        return NaN;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return Inf;
-      }
-    }
-    elsif ($e1->is_negative_infinite) {
-      if (defined $e2->dv) {
-        return negativeInf;
-      }
-      elsif ($e2->is_positive_infinite) {
-        return negativeInf;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return NaN;
-      }
-    }
-  }
-  elsif ($e1->is_integer) {
-    return integer($e1->iv + $e2->iv);
-  }
-  elsif ($e1->is_logical) {
-    return integer($e1->iv + $e2->iv);
-  }
-  else {
-    croak "Invalid type";
-  }
-}
-
-sub multiply {
-  my ($e1, $e2) = @_;
-  
-  return NA if $e1->is_na || $e2->is_na;
-  
-  if ($e1->is_character) {
-    croak "Error in a + b : non-numeric argument to binary operator";
-  }
-  elsif ($e1->is_complex) {
-    my $re = subtract(multiply($e1->re, $e2->re), multiply($e1->im, $e2->im));
-    my $im = add(multiply($e1->re, $e2->im), multiply($e1->im, $e2->re));
-    
-    return complex_double($re, $im);
-  }
-  elsif ($e1->is_double) {
-    return NaN if $e1->is_nan || $e2->is_nan;
-    if (defined $e1->dv) {
-      if (defined $e2->dv) {
-        my $value = $e1->dv * $e2->dv;
-        if ($value == $perl_inf_result) {
-          return Inf;
-        }
-        elsif ($value == $perl_negative_inf_result) {
-          return negativeInf;
-        }
-        else {
-          return double($value)
-        }
-      }
-      elsif ($e2->is_positive_infinite) {
-        if ($e1->dv == 0) {
-          return NaN;
-        }
-        elsif ($e1->dv > 0) {
-          return Inf;
-        }
-        elsif ($e1->dv < 0) {
-          return negativeInf;
-        }
-      }
-      elsif ($e2->is_negative_infinite) {
-        if ($e1->dv == 0) {
-          return NaN;
-        }
-        elsif ($e1->dv > 0) {
-          return negativeInf;
-        }
-        elsif ($e1->dv < 0) {
-          return Inf;
-        }
-      }
-    }
-    elsif ($e1->is_positive_infinite) {
-      if (defined $e2->dv) {
-        if ($e2->dv == 0) {
-          return NaN;
-        }
-        elsif ($e2->dv > 0) {
-          return Inf;
-        }
-        elsif ($e2->dv < 0) {
-          return negativeInf;
-        }
-      }
-      elsif ($e2->is_positive_infinite) {
-        return Inf;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return negativeInf;
-      }
-    }
-    elsif ($e1->is_negative_infinite) {
-      if (defined $e2->dv) {
-        if ($e2->dv == 0) {
-          return NaN;
-        }
-        elsif ($e2->dv > 0) {
-          return negativeInf;
-        }
-        elsif ($e2->dv < 0) {
-          return Inf;
-        }
-      }
-      elsif ($e2->is_positive_infinite) {
-        return negativeInf;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return Inf;
-      }
-    }
-  }
-  elsif ($e1->is_integer) {
-    return integer($e1->iv * $e2->iv);
-  }
-  elsif ($e1->is_logical) {
-    return integer($e1->iv * $e2->iv);
-  }
-  else {
-    croak "Invalid type";
-  }
-}
-
-sub divide {
-  my ($e1, $e2) = @_;
-  
-  return NA if $e1->is_na || $e2->is_na;
-  
-  if ($e1->is_character) {
-    croak "Error in a + b : non-numeric argument to binary operator";
-  }
-  elsif ($e1->is_complex) {
-    my $v3 = multiply($e1, Conj($e2));
-    my $abs2 = double($e2->re->value ** 2 + $e2->im->value ** 2);
-    my $re = divide($v3->re, $abs2);
-    my $im = divide($v3->im, $abs2);
-    
-    return complex_double($re, $im);
-  }
-  elsif ($e1->is_double) {
-    return NaN if $e1->is_nan || $e2->is_nan;
-    if (defined $e1->dv) {
-      if ($e1->dv == 0) {
-        if (defined $e2->dv) {
-          if ($e2->dv == 0) {
-            return NaN;
-          }
-          else {
-            return double(0)
-          }
-        }
-        elsif ($e2->is_infinite) {
-          return double(0);
-        }
-      }
-      elsif ($e1->dv > 0) {
-        if (defined $e2->dv) {
-          if ($e2->dv == 0) {
-            return Inf;
-          }
-          else {
-            my $value = $e1->dv / $e2->dv;
-            if ($value == $perl_inf_result) {
-              return Inf;
-            }
-            elsif ($value == $perl_negative_inf_result) {
-              return negativeInf;
-            }
-            else {
-              return double($value)
-            }
-          }
-        }
-        elsif ($e2->is_infinite) {
-          return double(0);
-        }
-      }
-      elsif ($e1->dv < 0) {
-        if (defined $e2->dv) {
-          if ($e2->dv == 0) {
-            return negativeInf;
-          }
-          else {
-            return double($e1->dv / $e2->dv);
-          }
-        }
-        elsif ($e2->is_infinite) {
-          return double(0);
-        }
-      }
-    }
-    elsif ($e1->is_positive_infinite) {
-      if (defined $e2->dv) {
-        if ($e2->dv >= 0) {
-          return Inf;
-        }
-        elsif ($e2->dv < 0) {
-          return negativeInf;
-        }
-      }
-      elsif ($e2->is_infinite) {
-        return NaN;
-      }
-    }
-    elsif ($e1->is_negative_infinite) {
-      if (defined $e2->dv) {
-        if ($e2->dv >= 0) {
-          return negativeInf;
-        }
-        elsif ($e2->dv < 0) {
-          return Inf;
-        }
-      }
-      elsif ($e2->is_infinite) {
-        return NaN;
-      }
-    }
-  }
-  elsif ($e1->is_integer) {
-    if ($e1->iv == 0) {
-      if ($e2->iv == 0) {
-        return NaN;
-      }
-      else {
-        return double(0);
-      }
-    }
-    elsif ($e1->iv > 0) {
-      if ($e2->iv == 0) {
-        return Inf;
-      }
-      else  {
-        return double($e1->iv / $e2->iv);
-      }
-    }
-    elsif ($e1->iv < 0) {
-      if ($e2->iv == 0) {
-        return negativeInf;
-      }
-      else {
-        return double($e1->iv / $e2->iv);
-      }
-    }
-  }
-  elsif ($e1->is_logical) {
-    if ($e1->iv == 0) {
-      if ($e2->iv == 0) {
-        return NaN;
-      }
-      elsif ($e2->iv == 1) {
-        return double(0);
-      }
-    }
-    elsif ($e1->iv == 1) {
-      if ($e2->iv == 0) {
-        return Inf;
-      }
-      elsif ($e2->iv == 1)  {
-        return double(1);
-      }
-    }
-  }
-  else {
-    croak "Invalid type";
-  }
-}
-
-sub raise {
-  my ($e1, $e2) = @_;
-  
-  return NA if $e1->is_na || $e2->is_na;
-  
-  if ($e1->is_character) {
-    croak "Error in a + b : non-numeric argument to binary operator";
-  }
-  elsif ($e1->is_complex) {
-    my $e1_c = Math::Complex->make($e1->re->value, $e1->im->value);
-    my $e2_c = Math::Complex->make($e2->re->value, $e2->im->value);
-    
-    my $v3_c;
-    if ($e2->re->value == 1/2 && $e2->im->value == 0) {
-      $v3_c = Math::Complex::sqrt($e1_c);
-    }
-    else {
-      $v3_c = $e1_c ** $e2_c;
-    }
-    my $re = Math::Complex::Re($v3_c);
-    my $im = Math::Complex::Im($v3_c);
-    
-    return complex($re, $im);
-  }
-  elsif ($e1->is_double) {
-    return NaN if $e1->is_nan || $e2->is_nan;
-    if (defined $e1->dv) {
-      if ($e1->dv == 0) {
-        if (defined $e2->dv) {
-          if ($e2->dv == 0) {
-            return double(1);
-          }
-          elsif ($e2->dv > 0) {
-            return double(0);
-          }
-          elsif ($e2->dv < 0) {
-            return Inf;
-          }
-        }
-        elsif ($e2->is_positive_infinite) {
-          return double(0);
-        }
-        elsif ($e2->is_negative_infinite) {
-          return Inf
-        }
-      }
-      elsif ($e1->dv > 0) {
-        if (defined $e2->dv) {
-          if ($e2->dv == 0) {
-            return double(1);
-          }
-          else {
-            my $value = $e1->dv ** $e2->dv;
-            if ($value == $perl_inf_result) {
-              return Inf;
-            }
-            elsif ($value == $perl_negative_inf_result) {
-              return negativeInf;
-            }
-            else {
-              return double($value)
-            }
-          }
-        }
-        elsif ($e2->is_positive_infinite) {
-          if ($e1->dv < 1) {
-            return double(0);
-          }
-          elsif ($e1->dv == 1) {
-            return double(1);
-          }
-          elsif ($e1->dv > 1) {
-            return Inf;
-          }
-        }
-        elsif ($e2->is_negative_infinite) {
-          if ($e1->dv < 1) {
-            return double(0);
-          }
-          elsif ($e1->dv == 1) {
-            return double(1);
-          }
-          elsif ($e1->dv > 1) {
-            return double(0);
-          }
-        }
-      }
-      elsif ($e1->dv < 0) {
-        if (defined $e2->dv) {
-          if ($e2->dv == 0) {
-            return double(-1);
-          }
-          else {
-            return double($e1->dv ** $e2->dv);
-          }
-        }
-        elsif ($e2->is_positive_infinite) {
-          if ($e1->dv > -1) {
-            return double(0);
-          }
-          elsif ($e1->dv == -1) {
-            return double(-1);
-          }
-          elsif ($e1->dv < -1) {
-            return negativeInf;
-          }
-        }
-        elsif ($e2->is_negative_infinite) {
-          if ($e1->dv > -1) {
-            return Inf;
-          }
-          elsif ($e1->dv == -1) {
-            return double(-1);
-          }
-          elsif ($e1->dv < -1) {
-            return double(0);
-          }
-        }
-      }
-    }
-    elsif ($e1->is_positive_infinite) {
-      if (defined $e2->dv) {
-        if ($e2->dv == 0) {
-          return double(1);
-        }
-        elsif ($e2->dv > 0) {
-          return Inf;
-        }
-        elsif ($e2->dv < 0) {
-          return double(0);
-        }
-      }
-      elsif ($e2->is_positive_infinite) {
-        return Inf;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return double(0);
-      }
-    }
-    elsif ($e1->is_negative_infinite) {
-      if (defined $e2->dv) {
-        if ($e2->dv == 0) {
-          return double(-1);
-        }
-        elsif ($e2->dv > 0) {
-          return negativeInf;
-        }
-        elsif ($e2->dv < 0) {
-          return double(0);
-        }
-      }
-      elsif ($e2->is_positive_infinite) {
-        return negativeInf;
-      }
-      elsif ($e2->is_negative_infinite) {
-        return double(0);
-      }
-    }
-  }
-  elsif ($e1->is_integer) {
-    if ($e1->iv == 0) {
-      if ($e2->iv == 0) {
-        return double(1);
-      }
-      elsif ($e2->iv > 0) {
-        return double(0);
-      }
-      elsif ($e2->iv < 0) {
-        return Inf;
-      }
-    }
-    elsif ($e1->iv > 0) {
-      if ($e2->iv == 0) {
-        return double(1);
-      }
-      else {
-        return double($e1->iv ** $e2->iv);
-      }
-    }
-    elsif ($e1->iv < 0) {
-      if ($e2->iv == 0) {
-        return double(-1);
-      }
-      else {
-        return double($e1->iv ** $e2->iv);
-      }
-    }
-  }
-  elsif ($e1->is_logical) {
-    if ($e1->iv == 0) {
-      if ($e2->iv == 0) {
-        return double(1);
-      }
-      elsif ($e2->iv == 1) {
-        return double(0);
-      }
-    }
-    elsif ($e1->iv ==  1) {
-      if ($e2->iv == 0) {
-        return double(1);
-      }
-      elsif ($e2->iv == 1) {
-        return double(1);
-      }
-    }
-  }
-  else {
-    croak "Invalid type";
-  }
-}
-
 sub remainder {
   my ($e1, $e2) = @_;
   
@@ -1908,15 +1306,6 @@ sub logical_to_integer {
   }
 }
 
-sub sqrt {
-  my $e1 = shift;
-  
-  my $type = $e1->type;
-  my $e2 = create($type, 1/2);
-  
-  return raise($e1, $e2);
-}
-
 sub more_than {
   my ($e1, $e2) = @_;
   
